@@ -202,11 +202,17 @@ class EventLoop(object):
                     continue
                 else:
                     logging.error(e)
+                    import traceback
+                    traceback.print_exc()
                     continue
             for handler in self._handlers:
-                # no exceptions should be raised by users
                 # TODO when there are a lot of handlers
-                handler(events)
+                try:
+                    handler(events)
+                except (OSError, IOError) as e:
+                    logging.error(e)
+                    import traceback
+                    traceback.print_exc()
 
 
 # from tornado
