@@ -67,77 +67,47 @@
 
 
 import time
-import threading
 import socket
 import logging
 import struct
+import errno
 import encrypt
 import eventloop
 import lru_cache
-import errno
+from common import parse_header
 
 
 BUF_SIZE = 65536
 
 
-def parse_header(data):
-    addrtype = ord(data[0])
-    dest_addr = None
-    dest_port = None
-    header_length = 0
-    if addrtype == 1:
-        if len(data) >= 7:
-            dest_addr = socket.inet_ntoa(data[1:5])
-            dest_port = struct.unpack('>H', data[5:7])[0]
-            header_length = 7
-        else:
-            logging.warn('[udp] header is too short')
-    elif addrtype == 3:
-        if len(data) > 2:
-            addrlen = ord(data[1])
-            if len(data) >= 2 + addrlen:
-                dest_addr = data[2:2 + addrlen]
-                dest_port = struct.unpack('>H', data[2 + addrlen:4 +
-                                          addrlen])[0]
-                header_length = 4 + addrlen
-            else:
-                logging.warn('[udp] header is too short')
-        else:
-            logging.warn('[udp] header is too short')
-    elif addrtype == 4:
-        if len(data) >= 19:
-            dest_addr = socket.inet_ntop(socket.AF_INET6, data[1:17])
-            dest_port = struct.unpack('>H', data[17:19])[0]
-            header_length = 19
-        else:
-            logging.warn('[udp] header is too short')
-    else:
-        logging.warn('unsupported addrtype %d' % addrtype)
-    if dest_addr is None:
-        return None
-    return (addrtype, dest_addr, dest_port, header_length)
-
-
 def client_key(a, b, c, d):
     return '%s:%s:%s:%s' % (a, b, c, d)
 
 
 class UDPRelay(object):
-    def __init__(self, listen_addr='127.0.0.1', listen_port=1080,
-                 remote_addr='127.0.0.1', remote_port=8387, password=None,
-                 method='table', timeout=300, is_local=True):
-        self._listen_addr = listen_addr
-        self._listen_port = listen_port
-        self._remote_addr = remote_addr
-        self._remote_port = remote_port
-        self._password = password
-        self._method = method
-        self._timeout = timeout
+    def __init__(self, config, is_local):
+        if is_local:
+            self._listen_addr = config['local_address']
+            self._listen_port = config['local_port']
+            self._remote_addr = config['server']
+            self._remote_port = config['server_port']
+        else:
+            self._listen_addr = config['server']
+            self._listen_port = config['server_port']
+            self._remote_addr = None
+            self._remote_port = None
+        self._password = config['password']
+        self._method = config['method']
+        self._timeout = config['timeout']
         self._is_local = is_local
-        self._cache = lru_cache.LRUCache(timeout=timeout,
+        self._cache = lru_cache.LRUCache(timeout=config['timeout'],
                                          close_callback=self._close_client)
-        self._client_fd_to_server_addr = lru_cache.LRUCache(timeout=timeout)
+        self._client_fd_to_server_addr = \
+            lru_cache.LRUCache(timeout=config['timeout'])
+        self._eventloop = None
         self._closed = False
+        self._last_time = time.time()
+        self._sockets = set()
 
         addrs = socket.getaddrinfo(self._listen_addr, self._listen_port, 0,
                                    socket.SOCK_DGRAM, socket.SOL_UDP)
@@ -152,6 +122,7 @@ class UDPRelay(object):
 
     def _close_client(self, client):
         if hasattr(client, 'close'):
+            self._sockets.remove(client.fileno())
             self._eventloop.remove(client)
             client.close()
         else:
@@ -198,6 +169,7 @@ class UDPRelay(object):
             else:
                 # drop
                 return
+            self._sockets.add(client.fileno())
             self._eventloop.add(client, eventloop.POLL_IN)
 
         data = data[header_length:]
@@ -239,7 +211,7 @@ class UDPRelay(object):
                 return
             # addrtype, dest_addr, dest_port, header_length = header_result
             response = '\x00\x00\x00' + data
-        client_addr = self._client_fd_to_server_addr.get(sock.fileno(), None)
+        client_addr = self._client_fd_to_server_addr.get(sock.fileno())
         if client_addr:
             self._server_socket.sendto(response, client_addr)
         else:
@@ -247,45 +219,29 @@ class UDPRelay(object):
             # simply drop that packet
             pass
 
-    def _run(self):
-        server_socket = self._server_socket
-        self._eventloop = eventloop.EventLoop()
-        self._eventloop.add(server_socket, eventloop.POLL_IN)
-        last_time = time.time()
-        while not self._closed:
-            try:
-                events = self._eventloop.poll(10)
-            except (OSError, IOError) as e:
-                if eventloop.errno_from_exception(e) == errno.EPIPE:
-                    # Happens when the client closes the connection
-                    continue
-                else:
-                    logging.error(e)
-                    continue
-            for sock, event in events:
-                if sock == self._server_socket:
-                    self._handle_server()
-                else:
-                    self._handle_client(sock)
-            now = time.time()
-            if now - last_time > 3.5:
-                self._cache.sweep()
-            if now - last_time > 7:
-                self._client_fd_to_server_addr.sweep()
-                last_time = now
-
-    def start(self):
+    def add_to_loop(self, loop):
         if self._closed:
-            raise Exception('closed')
-        t = threading.Thread(target=self._run)
-        t.setName('UDPThread')
-        t.setDaemon(False)
-        t.start()
-        self._thread = t
+            raise Exception('already closed')
+        self._eventloop = loop
+        loop.add_handler(self._handle_events)
+
+        server_socket = self._server_socket
+        self._eventloop.add(server_socket,
+                            eventloop.POLL_IN | eventloop.POLL_ERR)
+
+    def _handle_events(self, events):
+        for sock, fd, event in events:
+            if sock == self._server_socket:
+                self._handle_server()
+            elif sock and (fd in self._sockets):
+                self._handle_client(sock)
+        now = time.time()
+        if now - self._last_time > 3.5:
+            self._cache.sweep()
+        if now - self._last_time > 7:
+            self._client_fd_to_server_addr.sweep()
+            self._last_time = now
 
     def close(self):
         self._closed = True
         self._server_socket.close()
-
-    def thread(self):
-        return self._thread
